program RobInExample;
  { (c) 2000, Tom Verhoeff (TUE) }
  { Simple example for using RobIn in a strict input validator }

  { Format definition:

    The first line of file 'EXAMPLE.IN' contains two integers N and K
    with  2 <= N <= 9  and  1 <= K <= N.  The second line contains
    N lowerercase letters ('a'..'z', contiguous).
    The following K lines each contain one nonempty string of at most
    20 uppercase letters ('A'..'Z').
  }

uses RobIn;

const
  TaskPath  = '';
  ProgName  = 'Example';
  InpName   = ProgName + '.in';
  LogName   = ProgName + '.log';
  MdfName   = ''; { no message definition file }

  { bounds }
  MinN      =   2;
  MaxN      =   9;
  MinK      =   1;
  MaxK      =   9;
  MinLenS   =   3;
  MaxLenS   =  20;

var
  { input variables }
  N: MinN .. MaxN;
  K: MinK .. MaxK;
  L: array [ 1 .. MaxN ] of Char;
     { L[i] is i-th letter on second line }
  S: array [ 1 .. MaxK ] of String [ MaxLenS ];
     { S[j] is string on line 2+j }

  { auxiliary variables }
  i: 1 .. MaxN; { to traverse the N characters on second line }
  j: 1 .. MaxK; { to traverse the K lines after first two }

begin
  writeln ( ProgName, ': Validator for ', TaskPath + InpName )

; BeginLogging ( logFile, LogName,
                 OrganizerDiagnostics, UnlimitedReporting,
                 '', MdfName )

; OpenRIF ( inFile, InpName, InpName, StrictChecking )

  { read first line }
; N := ReadIntegerRIF ( inFile, Separated, 'N', 2, 9 )
; K := ReadIntegerRIF ( inFile, Separated, 'K', 1, N )
; ReadlnRIF ( inFile )

  { read second line }
; for i := 1 to N do begin
    L[i] := ReadCharRIF ( inFile, Contiguous,
                          'L[' + IntToStr(i,1) + ']',
                          LowerCaseChars )
  end { for i }
; ReadlnRIF ( inFile )

  { read following K lines }
; for j := 1 to K do begin
    S[j] := ReadStringRIF ( inFile, Contiguous,
                            'S[' + IntToStr(j,1) + ']',
                            UpperCaseChars, 1, 20 )
  ; ReadlnRIF ( inFile )
  end { for j }

; CloseRIF ( inFile )

  { possibly do some further analysis of the input variables here ... }

; EndLogging ( LogName <> '' )
end.