unit FileUtil;

interface

const
  inp: string[6] = 'INPUT';
  out: string[6] = 'OUTPUT';
  txt: string[4] = '.TXT';
  bak: string[4] = '.BAK';
  exe: string[4] = '.EXE';

  command: string = 'C:\DOS\COMMAND.COM';

procedure ReportDosError(const msg: string; d: integer);
procedure ReportExitCode(e: word);
function  ExistsF(const fname: string): boolean;
procedure BackUp(const fname: string);
procedure RenameF(const src, dest: string; bu: boolean);
procedure CopyF(const src, dest: string; bu: boolean);
procedure ShowTop(var f: text; const fname: string; k: integer; verbose: boolean);
procedure ShowPaged(const fname: string; p, offset: integer);
procedure GetFileSize(const fname: string; var s: longint);
procedure TimedExec(const prg, cmd: string; var rt: longint);

implementation

uses Dos, Crt, Timers;

{$UNDEF DOScopy}

const Test = false;

procedure ReportDosError(const msg: string; d: integer);
  var ta: word;
  begin
  ta := TextAttr ; HighVideo ; TextBackground(Red) ;
  write(msg) ;
  if d <> 0 then begin
    if msg <> '' then write(' ') ;
    write('DOS error code = ', d:1) ;
    case d of
       2: write(' (File not found)') ;
       3: write(' (Path not found)') ;
       4: write(' (Too many open files)') ;
       5: write(' (Access denied)') ;
       6: write(' (Invalid file handle)') ;
       8: write(' (Not enough memory)') ;
      10: write(' (Invalid environment)') ;
      11: write(' (Invalid format)') ;
      end { case }
    end { d<>0 } ;
  TextAttr := ta ;
  writeln
  end { ReportDosError } ;

procedure ReportExitCode(e: word);
  var ta: word; s: string;
  begin
  if e <> 0 then begin
    ta := TextAttr ; HighVideo ; TextBackground(Red) ;
    write('[') ;
    if Lo(e) <> 0 then begin
      write('exit code = ', Lo(e):1) ;
      s := ', '
      end
    else s := '' ;
    if Hi(e) <> 0 then begin
      write(s, ' terminated by ') ;
      case Hi(e) of
        1: write('Ctrl+C') ;
        2: write('device error') ;
        3: write('Keep procedure') ;
        end { case }
      end { if } ;
    write(']') ;
    TextAttr := ta ;
    write(' ')
    end { if }
  end { ReportExitCode } ;

function ExistsF(const fname: string): boolean;
  var f: text; ior: integer;
  begin
  Assign(f, fname) ; {$I-} reset(f) {$I+} ; ior := IOResult ;
  if ior <> 0 then
    ReportDosError('Cannot open "'+fname+'":', ior) ;
  ExistsF := (ior = 0)
  end { ExistsF } ;

procedure BackUp(const fname: string);
  { Create backup of file named fname+txt (if present) to fname+bak }
  var f, g: text; ior: integer;
  begin
  if txt = bak then begin
    writeln('Cannot backup ', bak, ' files') ;
    halt
    end { if } ;
  Assign(f, fname+txt) ; {$I-} reset(f) {$I+} ; ior := IOResult ;
  if ior <> 0 then { fname+txt not found }
    Exit ;
  writeln('Backing up "', fname+txt, '" to "', fname+bak, '"') ;
  Assign(g, fname+bak) ; {$I-} reset(g) {$I+} ; ior := IOResult ;
  if ior = 0 then begin { fname+bak found, delete it without asking }
    close(g) ;
    Erase(g)
    end { if } ;
  close(f) ;
  Rename(f, fname+bak)
  end { BackUp } ;

procedure RenameF(const src, dest: string; bu: boolean);
  { Move file named src+txt to dest+txt }
  var f: text;
  begin
  if bu then BackUp(dest) ;
  writeln('Renaming "', src+txt, '" to "', dest+txt, '"') ;
  Assign(f, src+txt) ; Rename(f, dest+txt) ;
  if DosError <> 0 then begin
    ReportDosError('Rename failed:', DosError) ;
    halt
    end { if }
  end { RenameF } ;

procedure CopyF(const src, dest: string; bu: boolean);
  { Copy file named src+txt to dest+txt }
  { pre: lines in src+txt are at most 255 characters }
  var f, g: text; ior: integer; s: string;
  begin
  Assign(f, src+txt) ; {$I-} reset(f) {$I+} ; ior := IOResult ;
  if ior <> 0 then begin { src+txt not found }
    ReportDosError('Cannot open "'+src+txt+'":', ior) ;
    halt
    end { if } ;
  { src+txt exists }
  if bu then BackUp(dest) ;
  Assign(g, dest+txt) ; rewrite(g) ; { dest.txt deleted without asking }
  writeln('Copying "', src+txt, '" to "', dest+txt, '"') ;
{$IFDEF DOScopy}
  close(f) ; close(g) ;
  if Test then writeln('Using DOS COPY') ;
  Exec(command, '/c copy ' + src+txt + ' ' + dest+txt) ;
  if DosError <> 0 then begin
    ReportDosError('Copy failed:', DosError) ;
    halt
    end { if } ;
  if DosExitCode <> 0 then halt
{$ELSE}
  if Test then writeln('Using internal COPY') ;
{$R-,I-,S-}
  while not eof(f) do begin
    readln(f, s) ;
    writeln(g, s)
    end { while } ;
{$R+,I+}
  close(f) ; close(g)
{$ENDIF}
  end { CopyF } ;

procedure ShowTop(var f: text; const fname: string; k: integer; verbose: boolean);
  { Write to f the topmost k lines of file named fname, all lines if k < 0 }
  var g: text; s: string; ior: integer;
  begin
  if k <> 0 then begin
    Assign(g, fname) ; {$I-} reset(g) {$I-} ; ior := IOResult ;
    if ior <> 0 then begin
      ReportDosError('ShowTop "'+fname+'" failed:', ior) ;
      Exit
      end { if } ;
    if verbose then begin
      if k < 0 then write(f, 'Contents')
      else if k = 1 then write(f, 'Topmost line')
      else write(f, 'Topmost ', k:1, ' lines') ;
      writeln(f, ' of "', fname, '":') ;
      end { if } ;
    while not eof(g) and (k <> 0) do begin
      readln(g, s) ;
      writeln(f, s) ;
      dec(k)
      end { while } ;
    close(g)
    end { if }
  end { ShowTop } ;

procedure ShowLine(m: longint; w: integer; s: string);
  { pre: m = line number, w = field width, s = line }
  var ta: word; i, j: integer;
  begin
  for i := 1 to Length(s) do
    if not (s[i] in [' '..'~']) then s[i] := chr(254) ;
  i := 0 ; { s[1..i] has been written }
  j := 79 - w ; if w > 0 then dec(j) ;
  { s is written in batches of at most j characters }
  repeat
    if i = 0 then begin
      if w > 0 then write(m:w, ' ')
      end { if i=0 }
    else write(' ':w, ' ') ;
    ta := TextAttr ; HighVideo ; TextBackground(Blue) ;
    write(copy(s, i+1, j)) ;
    TextAttr := ta ;
    writeln ;
    inc(i, j)
    until i >= Length(s)
  end { ShowLine } ;

function plural(n: longint): string;
  begin
  if n = 1 then plural := '' else plural := 's'
  end { plural } ;

procedure ShowPaged(const fname: string; p, offset: integer);
  { Write file named fname to screen.  Put line numbers in left column. }
  { Put contents in bright white on blue background. }
  { If p > 0 then the output is paged with page length p. }
  { If p = 0 then the output is not paged. }
  { If p < 0 then only the last -p lines are shown. }
  { When paging, offset (>0) is the number of lines shown initially. }
  { RESTRICTION: Long lines are wrapped, but count only for 1 screen line. }
  { RESTRICTION: Binary files are just dumped to the screen! }
  var
    g: text; s: string; ior, w: integer;
    m, n: longint;

  procedure Pause(b: boolean);
    { pre: b == "inside file", not b == "at end of file" }
    const ETX = chr(3); CR = chr(13); ESC = chr(27);
    var ch: char; ta: word;
    begin
    ta := TextAttr ; TextColor(Yellow) ; HighVideo ;
    if b then
      write('SPACE shows more, ESC shows end, ^C aborts:')
    else
      write('SPACE/ESC continues, ^C aborts: ') ;
    TextAttr := ta ;
    ch := ReadKey ;
    while not (ch in [ETX, ESC, ' ']) do begin
      if ch = chr(0) then ch := ReadKey ; { skip function key }
      ch := ReadKey
      end { while } ;
    write(CR) ; ClrEol ; { Erase prompt }
    case ch of
      ETX: halt ;
      ESC: if b then begin
        if m+p < n then writeln(':':w, ' [ skipped ]') ;
        p := -p
        end { ESC / if b } ;
      ' ': { nothing } ;
      end { case }
    end { Pause } ;

  begin { ShowPaged }
  Assign(g, fname) ; {$I-} reset(g) {$I-} ; ior := IOResult ;
  if ior <> 0 then begin
    ReportDosError('ShowPaged "'+fname+'" failed:', ior) ;
    halt
    end { if } ;
  write('Contents of "', fname, '"') ;
  n := 0 ; { # lines read }
  while not eof(g) do begin
    readln(g) ;
    inc(n)
    end { while } ;
  { n = # lines in g }
  m := 1 ; w := 0 ; { m = 10^w }
  while m <= n do begin m := 10*m ; inc(w) end ;
  { line numbers can be written with w digits }
  writeln(' (', n:1, ' line', plural(n), '):') ;
  reset(g) ;
  m := 0 ; { # lines shown }
  while not eof(g) do begin
    if (p > 0) and (m mod p = offset mod p) and (m <> 0) then
      Pause(true) ;
    readln(g, s) ; inc(m) ;
    if (p >= 0) or (m > n+p) then ShowLine(m, w, s)
    end { while } ;
  close(g) ;
  writeln('End of "', fname, '" (', n:1, ' line', plural(n), ')') ;
  Pause(false)
  end { ShowPaged } ;

procedure GetFileSize(const fname: string; var s: longint);
  var sr: SearchRec;
  begin
  FindFirst(fname, AnyFile, sr) ;
  if DosError <> 0 then begin
    ReportDosError('GetFileSize "'+fname+'" failed:', DosError) ;
    s := -DosError ;
    Exit
    end { if } ;
  s := sr.Size
  end { GetFileSize } ;

procedure TimedExec(const prg, cmd: string; var rt: longint);
  var ta: word; s: string; T: Timer; sec: longint; f: integer;
  begin
  if cmd = '' then s := '' else s := ' ' ;
  ta := TextAttr ; HighVideo ; TextBackground(Blue) ;
  write('Timing "', prg+s+cmd, '"') ;
  TextAttr := ta ;
  write(' ... ') ;
  SwapVectors ;
  T.Start ;
  Exec(prg+exe, cmd) ;
  T.Stop ;
  SwapVectors ;
  if DosError <> 0 then begin
    ReportDosError('Exec failed:', DosError) ;
    rt := -DosError ;
    Exit
    end { if } ;
  ReportExitCode(DosExitCode) ;
  rt := T.Elapsed(false) ;
  sec := rt div 100 ; f := rt mod 100 ;
  ta := TextAttr ; HighVideo ; TextBackground(Blue) ;
  write('Run time = ', sec:1, '.', f div 10:1, f mod 10:1, ' seconds') ;
  TextAttr := ta ;
  writeln
  end { TimedExec } ;

begin
end.